//--------------------------------------------------------------
// PlanarYUV16ToARGB.fx
//
// Copyright (c) LWKS Software Ltd.  All Rights Reserved
//--------------------------------------------------------------
DeclareLightworksEffect( PlanarYUV16ToARGB, Internal, Colour, kNoNotes, kNoFlags );

//--------------------------------------------------------------
// Inputs
//--------------------------------------------------------------
DeclareInput( Y, Point );
DeclareInput( Cr, Point );
DeclareInput( Cb, Point );

DeclareFloat3Param( ccCoefsAdd1 );
DeclareFloat3x3Param( ccCoefsMul2 );
DeclareFloat3Param( ccCoefsAdd3 );
DeclareFloatParam( Width );
DeclareFloatParam( Normalise );

//--------------------------------------------------------------
// Code
//--------------------------------------------------------------
DeclareEntryPoint()
{
   // we need to follow the surface order A16B16G16R16 LE ORDER
   // for each float pixel (x,y) we read word 1 to 4 of each plane
   int Yx = int( fmod(uv1.x * Width, 4) );
   // repeat 2 Chroma samples for each Y
   int Cx = int( fmod(uv1.x * Width / 2, 4) );

   float3 yuv;

   // we need to follow the surface order A16B16G16R16 LE ORDER
   // for each float pixel (x,y) we read word 1 to 4 of each plane
#ifdef WINDOWS
   yuv.r = tex2D(Y, uv1)[ Yx ];
   yuv.b = tex2D(Cr, uv2)[ Cx ];
   yuv.g = tex2D(Cb, uv3)[ Cx ];
#else
   // handle different pixel orders on Mac
   int4 map = int4( 2, 1, 0, 3 );

   yuv.r = tex2D(Y, uv1)[ map[ Yx ] ];
   yuv.b = tex2D(Cr, uv2)[ map[ Cx ] ];
   yuv.g = tex2D(Cb, uv3)[ map[ Cx ] ];
#endif

   float4 ret;

   // normalise 10/12 bit etc.
   yuv.rgb *= Normalise;

   ret.rgb = mul(yuv + ccCoefsAdd1, ccCoefsMul2) + ccCoefsAdd3;

   ret.a = 1;

   return ret;
}
