
struct VSO_2Tex
{
   float2 uv1 : TEXCOORD1;
   float2 uv2 : TEXCOORD2;
};

texture  gSourceImage;              // The source image
int      pSourceImageBitsPerComponent;

//--------------------------------------------------------------------------------------
// Texture samplers
//--------------------------------------------------------------------------------------
sampler SourceImageSampler =
sampler_state
{
   Texture = <gSourceImage>;
   MipFilter = POINT;
   MinFilter = POINT;
   MagFilter = POINT; //GAUSSIANQUAD; //POINT; //LINEAR;
   AddressU  = ClampToEdge;
   AddressV  = ClampToEdge;
};

DeclareFloat3x3Param( coefsMul );
DeclareFloat3Param( coefsAdd );
DeclareFloat3Param( coefsMulY );
DeclareFloatParam( coefsAddY );

#define Half_10Bit (0.5/1024.0)

//------------------------------------------------------------------------
float4 ps_main( VSO_2Tex In ) : COLOR0
{
   // U and V (Cr and Cb) samples are co-sited with odd-numbered Y-samples (eg, 1st, 3rd, 5th)

   float4 ret;

   float4 inDiffuse1;
   float4 inDiffuse2;

#ifdef WINDOWS
   if ( pSourceImageBitsPerComponent == 16 )
   {
      inDiffuse1 = tex2D(SourceImageSampler, In.uv2);
      inDiffuse2 = tex2D(SourceImageSampler, In.uv1); //3
   }
   else
#endif
   {
      inDiffuse1 = tex2D(SourceImageSampler, In.uv1);
      inDiffuse2 = tex2D(SourceImageSampler, In.uv2);
   }

   // First pixel (provides Y, Cr and Cb samples at this position)
   ret.bag = mul( inDiffuse1.rgb, coefsMul ) + coefsAdd;

   // Second pixel (Provides Y-sample at this position only)
   ret.r = mul( inDiffuse2.rgb, coefsMulY ) + coefsAddY;

   // Rounding correction
   ret += Half_10Bit;

   return ret;
}

technique T1 { pass P1 { PixelShader = compile PROFILE ps_main(); } }
