//--------------------------------------------------------------
// Blend.fx
//
// Copyright (c) LWKS Software Ltd.  All Rights Reserved
//--------------------------------------------------------------
#include "_utils.fx"

DeclareLightworksEffect( "Blend", "Mixes", "Components", kNoNotes, "CanSize" );

//--------------------------------------------------------------
// Params
//--------------------------------------------------------------
DeclareIntParam( SetTechnique, "Method", kNoGroup, 0, "In Front|Add|Subtract|Multiply|Screen|Overlay|Soft Light|Hard Light|Exclusion|Lighten|Darken|Average|Difference|Colour|Luminosity|Dodge|Burn" );
DeclareFloatParam( Opacity, "Fg Opacity", kNoGroup, kNoFlags, 1.0, 0.0, 1.0 );

//--------------------------------------------------------------
// Inputs
//--------------------------------------------------------------
DeclareInputs( fg, bg );
DeclareMask;

//--------------------------------------------------------------
// Code
//--------------------------------------------------------------
DeclareEntryPoint( InFront )
{
   float4 fgPix = ReadPixel( fg, uv1 );
   float4 bgPix = ReadPixel( bg, uv2 );

   float4 ret = lerp( bgPix, fgPix, fgPix.a * Opacity );
   ret.a = 1.0;

   return lerp( fgPix, ret, tex2D( Mask, uv1) );
}
//--------------------------------------------------------------
DeclareEntryPoint( Add )
{
   float4 fgPix = ReadPixel( fg, uv1 );
   float4 bgPix = ReadPixel( bg, uv2 );

   float4 ret = lerp( bgPix, clamp( fgPix + bgPix, 0.0, 1.0 ), fgPix.a * Opacity );
   ret.a = 1.0;

   return lerp( fgPix, ret, tex2D( Mask, uv1) );
}
//--------------------------------------------------------------
DeclareEntryPoint( Subtract )
{
   float4 fgPix = ReadPixel( fg, uv1 );
   float4 bgPix = ReadPixel( bg, uv2 );

   float4 ret = lerp( bgPix, clamp( fgPix - bgPix, 0.0, 1.0 ), fgPix.a * Opacity );
   ret.a = 1.0;

   return lerp( fgPix, ret, tex2D( Mask, uv1) );
}
//--------------------------------------------------------------
DeclareEntryPoint( Multiply )
{
   float4 fgPix = ReadPixel( fg, uv1 );
   float4 bgPix = ReadPixel( bg, uv2 );

   float4 ret = lerp( bgPix, bgPix * fgPix, fgPix.a * Opacity );
   ret.a = 1.0;

    return lerp( fgPix, ret, tex2D( Mask, uv1) );
}
//--------------------------------------------------------------//
DeclareEntryPoint( Screen )
{
   float4 fgPix = ReadPixel( fg, uv1 );
   float4 bgPix = ReadPixel( bg, uv2 );

   float4 ret = 1.0 -( ( 1.0 - fgPix ) * ( 1.0 - bgPix ) );
   ret = lerp( bgPix, ret, fgPix.a * Opacity );
   ret.a = 1.0;

   return lerp( fgPix, ret, tex2D( Mask, uv1) );
}
//--------------------------------------------------------------
DeclareEntryPoint( Overlay )
{
   float4 fgPix = ReadPixel( fg, uv1 );
   float4 bgPix = ReadPixel( bg, uv2 );

   float4 result;

   if ( bgPix.r < 0.5 )
      result.r = 2.0 * fgPix.r * bgPix.r;
   else
      result.r = 1.0 - ( 2.0 * ( 1.0 - fgPix.r ) * ( 1.0 - bgPix.r ) );

   if ( bgPix.g < 0.5 )
      result.g = 2.0 * fgPix.g * bgPix.g;
   else
      result.g = 1.0 - ( 2.0 * ( 1.0 - fgPix.g ) * ( 1.0 - bgPix.g ) );

   if ( bgPix.b < 0.5 )
      result.b = 2.0 * fgPix.b * bgPix.b;
   else
      result.b = 1.0 - ( 2.0 * ( 1.0 - fgPix.b ) * ( 1.0 - bgPix.b ) );

   result.rgb = lerp( bgPix.rgb, result.rgb, fgPix.a * Opacity );
   result.a   = 1.0;

   return lerp( fgPix, result, tex2D( Mask, uv1) );
}
//--------------------------------------------------------------
DeclareEntryPoint( SoftLight )
{
   float4 fgPix = ReadPixel( fg, uv1 );
   float4 bgPix = ReadPixel( bg, uv2 );

   float4 ret = ( 1.0 - bgPix ) * ( fgPix * bgPix ) + ( bgPix * ( 1.0 - ( ( 1.0 - bgPix ) * ( 1.0 - fgPix ) ) ) );

   ret = lerp( bgPix, ret, fgPix.a * Opacity );
   ret.a = 1.0;

   return lerp( fgPix, ret, tex2D( Mask, uv1) );
}
//--------------------------------------------------------------
DeclareEntryPoint( Hardlight )
{
   float4 fgPix = ReadPixel( fg, uv1 );
   float4 bgPix = ReadPixel( bg, uv2 );

   float4 result;

   if ( fgPix.r < 0.5 )
      result.r = 2.0 * fgPix.r * bgPix.r;
   else
      result.r = 1.0 - ( 2.0 * ( 1.0 - fgPix.r ) * ( 1.0 - bgPix.r ) );

   if ( fgPix.g < 0.5 )
      result.g = 2.0 * fgPix.g * bgPix.g;
   else
      result.g = 1.0 - ( 2.0 * ( 1.0 - fgPix.g ) * ( 1.0 - bgPix.g ) );

   if ( fgPix.b < 0.5 )
      result.b = 2.0 * fgPix.b * bgPix.b;
   else
      result.b = 1.0 - ( 2.0 * ( 1.0 - fgPix.b ) * ( 1.0 - bgPix.b ) );

   result.rgb = lerp( bgPix.rgb, result.rgb, fgPix.a * Opacity );
   result.a = 1.0;

   return lerp( fgPix, result, tex2D( Mask, uv1) );
}
//--------------------------------------------------------------
DeclareEntryPoint( Exclusion )
{
   float4 fgPix = ReadPixel( fg, uv1 );
   float4 bgPix = ReadPixel( bg, uv2 );

   float4 ret = fgPix + bgPix - ( 2.0 * fgPix * bgPix );
   ret = lerp( bgPix, ret, fgPix.a * Opacity );
   ret.a = 1.0;

   return lerp( fgPix, ret, tex2D( Mask, uv1) );
}
//--------------------------------------------------------------
DeclareEntryPoint( Lighten )
{
   float4 fgPix = ReadPixel( fg, uv1 );
   float4 bgPix = ReadPixel( bg, uv2 );

   float4 ret = lerp( bgPix, max( fgPix, bgPix ), fgPix.a * Opacity );
   ret.a = 1.0;

   return lerp( fgPix, ret, tex2D( Mask, uv1) );
}
//--------------------------------------------------------------
DeclareEntryPoint( Darken )
{
   float4 fgPix = ReadPixel( fg, uv1 );
   float4 bgPix = ReadPixel( bg, uv2 );

   float4 ret = min( fgPix, bgPix );
   ret = lerp( bgPix, ret, fgPix.a * Opacity );
   ret.a = 1.0;

   return lerp( fgPix, ret, tex2D( Mask, uv1) );
}
//--------------------------------------------------------------
DeclareEntryPoint( Average )
{
   float4 fgPix = ReadPixel( fg, uv1 );
   float4 bgPix = ReadPixel( bg, uv2 );

   float4 ret = ( fgPix + bgPix ) / 2.0;
   ret = lerp( bgPix, ret, fgPix.a * Opacity );
   ret.a = 1.0;

   return lerp( fgPix, ret, tex2D( Mask, uv1) );
}
//--------------------------------------------------------------
DeclareEntryPoint( Difference )
{
   float4 fgPix = ReadPixel( fg, uv1 );
   float4 bgPix = ReadPixel( bg, uv2 );

   float4 ret = abs( fgPix - bgPix );
   ret = lerp( bgPix, ret, fgPix.a * Opacity );
   ret.a = 1.0;

   return lerp( fgPix, ret, tex2D( Mask, uv1) );
}
/*
//--------------------------------------------------------------//
float4 Hue_main( float2 uv1 : TEXCOORD1, float2 uv2 : TEXCOORD2 ) : COLOR
{
   float4 fgPix = ReadPixel( fg, uv1 );
   float4 bgPix = ReadPixel( bg, uv2 );

   return fgPix;
}
//--------------------------------------------------------------//
float4 Saturation_main( float2 uv1 : TEXCOORD1, float2 uv2 : TEXCOORD2 ) : COLOR
{
   float4 fgPix = ReadPixel( fg, uv1 );
   float4 bgPix = ReadPixel( bg, uv2 );

   return fgPix;
}
*/
//--------------------------------------------------------------
DeclareEntryPoint( Color )
{
   float4 ret;
   float4 fgPix = ReadPixel( fg, uv1 );
   float4 bgPix = ReadPixel( bg, uv2 );

   // Calc source luminance but use dest colour..
   float dstY  = ( 0.257 * bgPix.r ) + ( 0.504 * bgPix.g ) + ( 0.098 * bgPix.b ) + 0.0625;
   float srcCr = ( 0.439 * fgPix.r ) - ( 0.368 * fgPix.g ) - ( 0.071 * fgPix.b ) + 0.5;
   float srcCb = ( -0.148 * fgPix.r ) -( 0.291 * fgPix.g ) + ( 0.439 * fgPix.b ) + 0.5;

   // Convert to RGB..
   float YBit = 1.164 * ( dstY - 0.0625 );
   ret.r = ( YBit + 1.596*(srcCr - 0.5) );
   ret.g = ( YBit - 0.813*(srcCr - 0.5) - 0.391*(srcCb - 0.5) );
   ret.b = ( YBit + 2.018*(srcCb - 0.5) );

   ret.rgb = lerp( bgPix.rgb, clamp( ret.rgb, 0.0, 1.0 ).rgb, fgPix.a * Opacity );
   ret.a = 1.0;

   return lerp( fgPix, ret, tex2D( Mask, uv1) );
}
//--------------------------------------------------------------
DeclareEntryPoint( Luminosity )
{
   float4 ret;
   float4 fgPix = ReadPixel( fg, uv1 );
   float4 bgPix = ReadPixel( bg, uv2 );

   // Calc source luminance but use dest colour..
   float srcY  = ( 0.257 * fgPix.r ) + ( 0.504 * fgPix.g ) + ( 0.098 * fgPix.b ) + 0.0625;
   float dstCr = ( 0.439 * bgPix.r ) - ( 0.368 * bgPix.g ) - ( 0.071 * bgPix.b ) + 0.5;
   float dstCb = ( -0.148 * bgPix.r ) -( 0.291 * bgPix.g ) + ( 0.439 * bgPix.b ) + 0.5;

   // Convert to RGB..
   float YBit = 1.164 * (srcY - 0.0625);
   ret.r = ( YBit + 1.596*(dstCr - 0.5) );
   ret.g = ( YBit - 0.813*(dstCr - 0.5) - 0.391*(dstCb - 0.5) );
   ret.b = ( YBit + 2.018*(dstCb - 0.5) );

   ret.rgb = lerp( bgPix.rgb, clamp( ret.rgb, 0.0, 1.0 ).rgb, fgPix.a * Opacity );
   ret.a = 1.0;

   return lerp( fgPix, ret, tex2D( Mask, uv1) );
}
//--------------------------------------------------------------
DeclareEntryPoint( Dodge )
{
   float4 fgPix = ReadPixel( fg, uv1 );
   float4 bgPix = ReadPixel( bg, uv2 );

   float4 ret = bgPix / ( 1.0 - fgPix );
   ret = lerp( bgPix, ret, fgPix.a * Opacity );
   ret.a = 1.0;

   return lerp( fgPix, ret, tex2D( Mask, uv1) );
}

//--------------------------------------------------------------
DeclareEntryPoint( Burn )
{
   float4 ret = float4( 0, 0, 0, 0 );

   float4 fgPix = ReadPixel( fg, uv1 );
   float4 bgPix = ReadPixel( bg, uv2 );

   float4 invbgPix = 1.0 - bgPix;

   if ( fgPix.r != 0 )
      ret.r = 1.0 - ( invbgPix.r / fgPix.r );

   if ( fgPix.g != 0 )
      ret.g = 1.0 - ( invbgPix.g / fgPix.g );

   if ( fgPix.b != 0 )
      ret.b = 1.0 - ( invbgPix.b / fgPix.b );

   ret = lerp( bgPix, clamp( ret, 0.0, 1.0 ), fgPix.a * Opacity );
   ret.a = 1.0;

   return lerp( fgPix, ret, tex2D( Mask, uv1) );
}
