//--------------------------------------------------------------//
// difference.fx
//
// Copyright (c) LWKS Software Ltd.  All Rights Reserved
//--------------------------------------------------------------//
DeclareLightworksEffect( "Difference", "Mixes", "Components", kNoNotes, "CanSize" );

//--------------------------------------------------------------
// Inputs
//--------------------------------------------------------------
DeclareInputs( fg, bg );

//--------------------------------------------------------------
// Params
//--------------------------------------------------------------
DeclareFloatParam( Amplify,   "Amplify",   kNoGroup, kNoFlags, 1, 1, 10 );
DeclareFloatParam( Threshold, "Threshold", kNoGroup, kNoFlags, 0, 0, 1 );
DeclareIntParam( SetTechnique, "View", kNoGroup, 0, "Difference|Difference over input one|Difference over input two|Input one|Input two|Input mix" );

float4 kWhite = float4( 1,1,1,1 );
float4 kBlack = float4( 0,0,0,1 );

//--------------------------------------------------------------
// Code
//--------------------------------------------------------------
DeclareEntryPoint( Difference )
{
   float4 fgPix = ReadPixel( fg, uv1 );
   float4 bgPix = ReadPixel( bg, uv2 );

   float4 ret = abs( fgPix - bgPix );

   if ( ( ( ret.r + ret.g + ret.b ) * 10 ) > Threshold )
   {
      ret  = min( ret * Amplify, kWhite );
      ret.a = 1.0;
   }
   else
      ret = kBlack;

   return ret;
}
//--------------------------------------------------------------
DeclareEntryPoint( Overlay1 )
{
   float4 fgPix = ReadPixel( fg, uv1 );
   float4 bgPix = ReadPixel( bg, uv2 );

   float4 ret = abs( fgPix - bgPix );

   if ( ( ( ret.r + ret.g + ret.b ) * 10 ) > Threshold )
   {
      ret  = kWhite;
      ret.a = 1.0;
   }
   else
      ret = fgPix;

   return ret;
}
//--------------------------------------------------------------
DeclareEntryPoint( Overlay2 )
{
   float4 fgPix = ReadPixel( fg, uv1 );
   float4 bgPix = ReadPixel( bg, uv2 );

   float4 ret = abs( fgPix - bgPix );

   if ( ( ( ret.r + ret.g + ret.b ) * 10 ) > Threshold )
   {
      ret  = kWhite;
      ret.a = 1.0;
   }
   else
      ret = bgPix;

   return ret;
}
//--------------------------------------------------------------
DeclareEntryPoint( Input1 )
{
   return ReadPixel( fg, uv1 );
}
//--------------------------------------------------------------
DeclareEntryPoint( Input2 )
{
   return ReadPixel( bg, uv2 );
}
//--------------------------------------------------------------
DeclareEntryPoint( Mix )
{
   float4 fgPix = ReadPixel( fg, uv1 );
   float4 bgPix = ReadPixel( bg, uv2 );

   return lerp( fgPix, bgPix, 0.5 );
}
